
!CodeHolder methodsFor: '*ToolBuilder-Specs' stamp: 'ar 2/12/2005 14:32'!
buildCodeProvenanceButtonWith: builder
	| buttonSpec |
	buttonSpec := builder pluggableActionButtonSpec new.
	buttonSpec model: self.
	buttonSpec label: #codePaneProvenanceString.
	buttonSpec action: #offerWhatToShowMenu.
	buttonSpec help: 'Governs what view is shown in the code pane.  Click here to change the view'.
	^buttonSpec! !

!CodeHolder methodsFor: '*ToolBuilder-Specs' stamp: 'ar 2/12/2005 14:32'!
buildOptionalButtonsWith: builder

	| panelSpec buttonSpec |
	panelSpec := builder pluggablePanelSpec new.
	panelSpec children: OrderedCollection new.
	self optionalButtonPairs do:[:spec|
		buttonSpec := builder pluggableActionButtonSpec new.
		buttonSpec model: self.
		buttonSpec label: spec first.
		buttonSpec action: spec second.
		spec second == #methodHierarchy ifTrue:[
			buttonSpec color: #inheritanceButtonColor.
		]. 
		spec size > 2 ifTrue:[buttonSpec help: spec third].
		panelSpec children add: buttonSpec.
	].
	"What to show"
	panelSpec children add: (self buildCodeProvenanceButtonWith: builder).

	panelSpec layout: #horizontal. "buttons"
	^panelSpec! !

!CodeHolder methodsFor: '*ToolBuilder-Specs-override' stamp: 'ar 2/12/2005 14:28'!
decorateButtons
	"Change screen feedback for any buttons in the UI of the receiver that may wish it.  Initially, it is only the Inheritance button that is decorated, but one can imagine others."
	self changed: #inheritanceButtonColor.
	self decorateForInheritance ! !

!CodeHolder methodsFor: '*ToolBuilder-Specs' stamp: 'ar 2/12/2005 14:27'!
inheritanceButtonColor
	"Check to see if the currently-viewed method has a super send or an override, and if so, change screen feedback, unless the #decorateBrowserButtons says not to."

	| flags aColor |
	((currentCompiledMethod isKindOf: CompiledMethod) and: [Preferences decorateBrowserButtons])
		ifFalse: [^Color transparent].

	"This table duplicates the old logic, but adds two new colors for the cases where there is a superclass definition, but this method doesn't call it."

	flags _ 0.
	self isThisAnOverride ifTrue: [ flags _ flags bitOr: 4 ].
	currentCompiledMethod sendsToSuper ifTrue: [ flags _ flags bitOr: 2 ].
	self isThereAnOverride ifTrue: [ flags _ flags bitOr: 1 ].
	aColor _ {
		Color transparent.
		Color tan lighter.
		Color green muchLighter.
		Color blue muchLighter.
		Color red muchLighter.	"has super but doesn't call it"
		(Color r: 0.94 g: 0.823 b: 0.673).	"has sub; has super but doesn't call it"
		Color green muchLighter.
		Color blue muchLighter.
	} at: flags + 1.

	^aColor! !

!Browser methodsFor: '*ToolBuilder-Specs' stamp: 'ar 2/12/2005 14:19'!
buildWith: builder
	"Create the ui for the browser"
	| windowSpec listSpec textSpec buttonSpec panelSpec max |
	windowSpec := builder pluggableWindowSpec new.
	windowSpec model: self.
	windowSpec label: 'System Browser'.
	windowSpec children: OrderedCollection new.

	max := self wantsOptionalButtons ifTrue:[0.43] ifFalse:[0.5].
	listSpec := builder pluggableListSpec new.
	listSpec 
		model: self;
		list: #systemCategoryList; 
		getIndex: #systemCategoryListIndex; 
		setIndex: #systemCategoryListIndex:; 
		menu: #systemCategoryMenu:; 
		keyPress: #systemCatListKey:from:;
		frame: (0@0 corner: 0.25@max).
	windowSpec children add: listSpec.

	listSpec := builder pluggableListSpec new.
	listSpec 
		model: self;
		list: #classList; 
		getIndex: #classListIndex; 
		setIndex: #classListIndex:; 
		menu: #classListMenu:; 
		keyPress: #classListKey:from:;
		frame: (0.25@0 corner: 0.5@(max-0.1)).
	windowSpec children add: listSpec.

	panelSpec := builder pluggablePanelSpec new.
	panelSpec frame: (0.25@(max-0.1) corner: 0.5@max).
	panelSpec children: OrderedCollection new.
	windowSpec children addLast: panelSpec.

		buttonSpec := builder pluggableButtonSpec new.
		buttonSpec 
			model: self;
			label: 'instance'; 
			state: #instanceMessagesIndicated; 
			action: #indicateInstanceMessages;
			frame: (0@0 corner: 0.4@1).
		panelSpec children addLast: buttonSpec.

		buttonSpec := builder pluggableButtonSpec new.
		buttonSpec 
			model: self;
			label: '?'; 
			state: #classCommentIndicated; 
			action: #plusButtonHit;
			frame: (0.4@0 corner: 0.6@1).
		panelSpec children addLast: buttonSpec.

		buttonSpec := builder pluggableButtonSpec new.
		buttonSpec 
			model: self;
			label: 'class'; 
			state: #classMessagesIndicated; 
			action: #indicateClassMessages;
			frame: (0.6@0 corner: 1@1).
		panelSpec children addLast: buttonSpec.

	listSpec := builder pluggableListSpec new.
	listSpec 
		model: self;
		list: #messageCategoryList; 
		getIndex: #messageCategoryListIndex; 
		setIndex: #messageCategoryListIndex:; 
		menu: #messageCategoryMenu:; 
		keyPress: #arrowKey:from:;
		frame: (0.5@0 corner: 0.75@max).
	windowSpec children add: listSpec.

	listSpec := builder pluggableListSpec new.
	listSpec 
		model: self;
		list: #messageList; 
		getIndex: #messageListIndex; 
		setIndex: #messageListIndex:; 
		menu: #messageListMenu:shifted:; 
		keyPress: #messageListKey:from:;
		frame: (0.75@0 corner: 1@max).
	windowSpec children add: listSpec.

	self wantsOptionalButtons ifTrue:[
		panelSpec := self buildOptionalButtonsWith: builder.
		panelSpec frame: (0@0.43 corner: 1@0.5).
		windowSpec children add: panelSpec.
	].

	textSpec := builder pluggableTextSpec new.
	textSpec 
		model: self;
		getText: #contents; 
		setText: #contents:notifying:; 
		selection: #contentsSelection; 
		menu: #codePaneMenu:shifted:;
		frame: (0@0.5corner: 1@1).
	windowSpec children add: textSpec.

	^builder build: windowSpec! !

!Debugger methodsFor: '*ToolBuilder-Specs' stamp: 'ar 2/12/2005 18:30'!
buildFullWith: builder
	| windowSpec listSpec textSpec panelSpec extent |
	windowSpec := builder pluggableWindowSpec new.
	windowSpec model: self.
	windowSpec label: 'Debugger'.
	Display height < 800 "a small screen" 
		ifTrue:[extent := RealEstateAgent standardWindowExtent]
		ifFalse:[extent := 600@700].
	windowSpec extent: extent.
	windowSpec children: OrderedCollection new.

	listSpec := builder pluggableListSpec new.
	listSpec 
		model: self;
		list: #contextStackList; 
		getIndex: #contextStackIndex; 
		setIndex: #toggleContextStackIndex:; 
		menu: #contextStackMenu:shifted:; 
		keyPress: #contextStackKey:from:;
		frame: (0@0 corner: 1@0.25).
	windowSpec children add: listSpec.


	panelSpec := self buildOptionalButtonsWith: builder.
	panelSpec frame: (0@0.25 corner: 1@0.3).
	windowSpec children add: panelSpec.

	textSpec := builder pluggableTextSpec new.
	textSpec 
		model: self;
		getText: #contents; 
		setText: #contents:notifying:; 
		selection: #contentsSelection; 
		menu: #codePaneMenu:shifted:;
		frame: (0@0.3corner: 1@0.8).
	windowSpec children add: textSpec.

	listSpec := builder pluggableListSpec new.
	listSpec 
		model: self receiverInspector;
		list: #fieldList; 
		getIndex: #selectionIndex; 
		setIndex: #toggleIndex:; 
		menu: #fieldListMenu:; 
		keyPress: #inspectorKey:from:;
		frame: (0@0.8 corner: 0.2@1).
	windowSpec children add: listSpec.

	textSpec := builder pluggableTextSpec new.
	textSpec 
		model: self receiverInspector;
		getText: #contents; 
		setText: #accept:; 
		selection: #contentsSelection; 
		menu: #codePaneMenu:shifted:;
		frame: (0.2@0.8 corner: 0.5@1).
	windowSpec children add: textSpec.

	listSpec := builder pluggableListSpec new.
	listSpec 
		model: self contextVariablesInspector;
		list: #fieldList; 
		getIndex: #selectionIndex; 
		setIndex: #toggleIndex:; 
		menu: #fieldListMenu:; 
		keyPress: #inspectorKey:from:;
		frame: (0.5@0.8 corner: 0.7@1).
	windowSpec children add: listSpec.

	textSpec := builder pluggableTextSpec new.
	textSpec 
		model: self contextVariablesInspector;
		getText: #contents; 
		setText: #accept:; 
		selection: #contentsSelection; 
		menu: #codePaneMenu:shifted:;
		frame: (0.7@0.8 corner: 1@1).
	windowSpec children add: textSpec.

	^builder build: windowSpec! !

!Debugger methodsFor: '*ToolBuilder-Specs' stamp: 'ar 2/12/2005 18:15'!
buildNotifierWith: builder label: label message: messageString
	| windowSpec listSpec textSpec panelSpec buttonSpec |
	windowSpec := builder pluggableWindowSpec new.
	windowSpec model: self.
	windowSpec extent: 450 @ 156. "nice and wide to show plenty of the error msg"
	windowSpec label: label.
	windowSpec children: OrderedCollection new.

	panelSpec := builder pluggablePanelSpec new.
	panelSpec children: OrderedCollection new.
	self preDebugButtonQuads do:[:spec|
		buttonSpec := builder pluggableButtonSpec new.
		buttonSpec model: self.
		buttonSpec label: spec first.
		buttonSpec action: spec second.
		buttonSpec help: spec fourth.
		panelSpec children add: buttonSpec.
	].
	panelSpec layout: #horizontal. "buttons"
	panelSpec frame: (0@0 corner: 1@0.2).
	windowSpec children add: panelSpec.

	Preferences eToyFriendly | messageString notNil ifTrue:[
		listSpec := builder pluggableListSpec new.
		listSpec 
			model: self;
			list: #contextStackList; 
			getIndex: #contextStackIndex; 
			setIndex: #debugAt:; 
			frame: (0@0.2 corner: 1@1).
		windowSpec children add: listSpec.
	] ifFalse:[
		textSpec := builder pluggableTextSpec new.
		textSpec 
			model: self;
			getText: #preDebugMessageString; 
			setText: nil; 
			selection: nil; 
			menu: #debugProceedMenu:;
			frame: (0@0.2corner: 1@1).
		windowSpec children add: textSpec.
	].

	^windowSpec! !

!Debugger methodsFor: '*ToolBuilder-Specs' stamp: 'ar 2/11/2005 16:24'!
buildWith: aBuilder
	^self buildFullWith: aBuilder! !

!Debugger methodsFor: '*ToolBuilder-Specs' stamp: 'ar 2/12/2005 18:25'!
openNotifierWith: builder label: label message: msgString
	| spec msg |
	Sensor flushKeyboard.
	savedCursor _ Sensor currentCursor.
	Sensor currentCursor: Cursor normal.
	(label beginsWith: 'Space is low')
		ifTrue: [msg _ self lowSpaceChoices, (msgString ifNil: [''])]
		ifFalse: [msg _ msgString].
	isolationHead ifNotNil:
		["We have already revoked the isolation layer -- now jump to the parent project."
		msg _ self isolationRecoveryAdvice, msgString.
		failedProject _ Project current.
		isolationHead parent enterForEmergencyRecovery].

	Display fullScreen.
	spec := self buildNotifierWith: builder label: label message: msg.
	builder open: spec.
	Smalltalk isMorphic ifTrue: [
		errorWasInUIProcess _ CurrentProjectRefactoring newProcessIfUI: interruptedProcess.
	].
! !

!Debugger methodsFor: '*ToolBuilder-Specs' stamp: 'ar 2/11/2005 16:25'!
preDebugMessageString
	^'An error has occurred; you should probably just hit ''abandon''.  Sorry!!'! !

!ChangeList methodsFor: '*ToolBuilder-Specs' stamp: 'ar 2/12/2005 13:50'!
buildWith: builder
	^self buildWith: builder multiSelect: true! !

!ChangeList methodsFor: '*ToolBuilder-Specs' stamp: 'ar 2/12/2005 14:46'!
buildWith: builder multiSelect: multiSelect 
	"Open a morphic view for the messageSet, whose label is labelString. 
	The listView may be either single or multiple selection type"

	| windowSpec max listSpec panelSpec textSpec |
	windowSpec := builder pluggableWindowSpec new.
	windowSpec model: self.
	windowSpec label: 'System Browser'.
	windowSpec children: OrderedCollection new.

	max := self wantsOptionalButtons ifTrue:[0.33] ifFalse:[0.4].

	multiSelect ifTrue:[
		listSpec := builder pluggableMultiSelectionListSpec new.
		listSpec getSelectionList: #listSelectionAt:.
		listSpec setSelectionList: #listSelectionAt:put:.
	] ifFalse:[
		listSpec := builder pluggableListSpec new.
	].

	listSpec 
		model: self;
		list: #list; 
		getIndex: #listIndex; 
		setIndex: #toggleListIndex:; 
		menu: (self showsVersions ifTrue: [#versionsMenu:] ifFalse: [#changeListMenu:]); 
		keyPress: #changeListKey:from:;
		frame: (0@0 corner: 1@max).
	windowSpec children add: listSpec.

	self wantsOptionalButtons ifTrue:[
		panelSpec := self buildOptionalButtonsWith: builder.
		panelSpec frame: (0@0.33 corner: 1@0.4).
		windowSpec children add: panelSpec.
	].

	textSpec := builder pluggableTextSpec new.
	textSpec 
		model: self;
		getText: #contents; 
		setText: #contents:;
		selection: #contentsSelection; 
		menu: #codePaneMenu:shifted:;
		frame: (0@0.4corner: 1@1).
	windowSpec children add: textSpec.

	^builder build: windowSpec! !

!ProcessBrowser methodsFor: '*ToolBuilder-Specs' stamp: 'ar 2/11/2005 20:38'!
buildWith: builder
	"Create a pluggable version of me, answer a window"
	| windowSpec listSpec textSpec |
	windowSpec := builder pluggableWindowSpec new.
	windowSpec model: self.
	windowSpec label: 'Process Browser'.
	windowSpec children: OrderedCollection new.

	listSpec := builder pluggableListSpec new.
	listSpec 
		model: self;
		list: #processNameList; 
		getIndex: #processListIndex; 
		setIndex: #processListIndex:; 
		menu: #processListMenu:; 
		keyPress: #processListKey:from:;
		frame: (0 @ 0 extent: 0.5 @ 0.5).
	windowSpec children add: listSpec.

	listSpec := builder pluggableListSpec new.
	listSpec 
		model: self;
		list: #stackNameList; 
		getIndex: #stackListIndex; 
		setIndex: #stackListIndex:; 
		menu: #stackListMenu:; 
		keyPress: #stackListKey:from:;
		frame: (0.5 @ 0.0 extent: 0.5 @ 0.5).
	windowSpec children add: listSpec.

	textSpec := builder pluggableTextSpec new.
	textSpec 
		model: self;
		getText: #selectedMethod; 
		setText: nil; 
		selection: nil; 
		menu: nil;
		frame: (0 @ 0.5 corner: 1 @ 1).
	windowSpec children add: textSpec.

	^builder build: windowSpec! !

!StringHolder methodsFor: '*ToolBuilder-Specs' stamp: 'ar 2/11/2005 20:36'!
buildWith: builder
	| windowSpec textSpec |
	windowSpec := builder pluggableWindowSpec new.
	windowSpec model: self.
	windowSpec label: 'Workspace'.
	windowSpec children: OrderedCollection new.
	textSpec := builder pluggableTextSpec new.
	textSpec 
		model: self;
		getText: #contents; 
		setText: #acceptContents:; 
		selection: nil; 
		menu: #codePaneMenu:shifted:;
		frame: (0@0corner: 1@1).
	windowSpec children add: textSpec.

	^builder build: windowSpec! !

!SimpleServiceEntry methodsFor: '*ToolBuilder-Specs' stamp: 'ar 2/12/2005 14:34'!
buildWith: builder in: aModel
	"Answer a button spec that will trigger the receiver service in aModel"
	| buttonSpec |
	buttonSpec := builder pluggableActionButtonSpec new.
	buttonSpec 
		model: self; 
		label: self buttonLabel;
		help: self description;
		action: (MessageSend receiver: aModel selector: #executeService: argument: self).
	^builder build: buttonSpec! !

!FileContentsBrowser methodsFor: '*ToolBuilder-Specs' stamp: 'ar 2/12/2005 14:20'!
buildWith: builder
	"Create the ui for the browser"
	| windowSpec listSpec textSpec buttonSpec panelSpec max |
	windowSpec := builder pluggableWindowSpec new.
	windowSpec model: self.
	windowSpec label: 'System Browser'.
	windowSpec children: OrderedCollection new.

	max := self wantsOptionalButtons ifTrue:[0.43] ifFalse:[0.5].
	listSpec := builder pluggableListSpec new.
	listSpec 
		model: self;
		list: #systemCategoryList; 
		getIndex: #systemCategoryListIndex; 
		setIndex: #systemCategoryListIndex:; 
		menu: #packageListMenu:; 
		keyPress: #packageListKey:from:;
		frame: (0@0 corner: 0.25@max).
	windowSpec children add: listSpec.

	listSpec := builder pluggableListSpec new.
	listSpec 
		model: self;
		list: #classList; 
		getIndex: #classListIndex; 
		setIndex: #classListIndex:; 
		menu: #classListMenu:; 
		keyPress: #classListKey:from:;
		frame: (0.25@0 corner: 0.5@(max-0.1)).
	windowSpec children add: listSpec.

	panelSpec := builder pluggablePanelSpec new.
	panelSpec frame: (0.25@(max-0.1) corner: 0.5@max).
	panelSpec children: OrderedCollection new.
	windowSpec children addLast: panelSpec.

		buttonSpec := builder pluggableButtonSpec new.
		buttonSpec 
			model: self;
			label: 'instance'; 
			state: #instanceMessagesIndicated; 
			action: #indicateInstanceMessages;
			frame: (0@0 corner: 0.4@1).
		panelSpec children addLast: buttonSpec.

		buttonSpec := builder pluggableButtonSpec new.
		buttonSpec 
			model: self;
			label: '?'; 
			state: #classCommentIndicated; 
			action: #plusButtonHit;
			frame: (0.4@0 corner: 0.6@1).
		panelSpec children addLast: buttonSpec.

		buttonSpec := builder pluggableButtonSpec new.
		buttonSpec 
			model: self;
			label: 'class'; 
			state: #classMessagesIndicated; 
			action: #indicateClassMessages;
			frame: (0.6@0 corner: 1@1).
		panelSpec children addLast: buttonSpec.

	listSpec := builder pluggableListSpec new.
	listSpec 
		model: self;
		list: #messageCategoryList; 
		getIndex: #messageCategoryListIndex; 
		setIndex: #messageCategoryListIndex:; 
		menu: #messageCategoryMenu:; 
		keyPress: #arrowKey:from:;
		frame: (0.5@0 corner: 0.75@max).
	windowSpec children add: listSpec.

	listSpec := builder pluggableListSpec new.
	listSpec 
		model: self;
		list: #messageList; 
		getIndex: #messageListIndex; 
		setIndex: #messageListIndex:; 
		menu: #messageListMenu:shifted:; 
		keyPress: #messageListKey:from:;
		frame: (0.75@0 corner: 1@max).
	windowSpec children add: listSpec.

	self wantsOptionalButtons ifTrue:[
		panelSpec := self buildOptionalButtonsWith: builder.
		panelSpec frame: (0@0.43 corner: 1@0.5).
		windowSpec children add: panelSpec.
	].

	textSpec := builder pluggableTextSpec new.
	textSpec 
		model: self;
		getText: #contents; 
		setText: #contents:notifying:; 
		selection: #contentsSelection; 
		menu: #codePaneMenu:shifted:;
		frame: (0@0.5corner: 1@0.92).
	windowSpec children add: textSpec.

	textSpec := builder pluggableInputFieldSpec new.
	textSpec 
		model: self;
		getText: #infoViewContents; 
		frame: (0@0.92corner: 1@1).
	windowSpec children add: textSpec.

	^builder build: windowSpec! !

!DualChangeSorter methodsFor: '*ToolBuilder-Specs' stamp: 'ar 2/11/2005 20:33'!
buildWith: builder
	| windowSpec |
	leftCngSorter _ ChangeSorter new myChangeSet: ChangeSet current.
	leftCngSorter parent: self.
	rightCngSorter _ ChangeSorter new myChangeSet: 
			ChangeSorter secondaryChangeSet.
	rightCngSorter parent: self.

	windowSpec := builder pluggableWindowSpec new.
	windowSpec model: self.
	windowSpec label: 'Change Sorter'.
	windowSpec children: OrderedCollection new.
	leftCngSorter buildWith: builder in: windowSpec rect: (0@0 extent: 0.5@1).
	rightCngSorter buildWith: builder in: windowSpec rect: (0.5@0 extent: 0.5@1).
	^builder build: windowSpec
! !

!MessageSet methodsFor: '*ToolBuilder-Specs' stamp: 'ar 2/12/2005 14:53'!
buildWith: builder
	| windowSpec max listSpec panelSpec textSpec |
	windowSpec := builder pluggableWindowSpec new.
	windowSpec model: self.
	windowSpec label: 'System Browser'.
	windowSpec children: OrderedCollection new.

	self wantsOptionalButtons ifTrue:[max := 0.3] ifFalse:[max := 0.2].
	listSpec := builder pluggableListSpec new.
	listSpec 
		model: self;
		list: #messageList; 
		getIndex: #messageListIndex; 
		setIndex: #messageListIndex:; 
		menu: #messageListMenu:shifted:; 
		keyPress: #messageListKey:from:;
		frame: (0@0 corner: 1@0.2).
	windowSpec children add: listSpec.

	self wantsOptionalButtons ifTrue:[
		panelSpec := self buildOptionalButtonsWith: builder.
		panelSpec frame: (0@0.2 corner: 1@max).
		windowSpec children add: panelSpec.
	].

	textSpec := builder pluggableTextSpec new.
	textSpec 
		model: self;
		getText: #contents; 
		setText: #contents:notifying:; 
		selection: #contentsSelection; 
		menu: #codePaneMenu:shifted:;
		frame: (0@max corner: 1@1).
	windowSpec children add: textSpec.

	^builder build: windowSpec! !

!Model class methodsFor: '*ToolBuilder-Specs' stamp: 'ar 2/11/2005 16:26'!
buildWith: toolBuilder
	^self new buildWith: toolBuilder! !

!ChangeSorter methodsFor: '*ToolBuilder-Specs' stamp: 'ar 2/11/2005 20:32'!
buildWith: builder
	"
		MorphicUIBuilder open: ChangeSorter.
	"
	|  windowSpec |
	windowSpec := builder pluggableWindowSpec new.
	windowSpec label: 'Change Sorter'.
	windowSpec model: self.
	windowSpec children: OrderedCollection new.
	self buildWith: builder in: windowSpec rect: (0@0 extent: 1@1).
	^builder build: windowSpec! !

!ChangeSorter methodsFor: '*ToolBuilder-Specs' stamp: 'ar 2/12/2005 16:45'!
buildWith: builder in: window rect: rect
	| csListHeight msgListHeight csMsgListHeight listSpec textSpec |
	contents _ ''.
	csListHeight _ 0.25.
	msgListHeight _ 0.25.
	csMsgListHeight _ csListHeight + msgListHeight.

	listSpec := builder pluggableListSpec new.
	listSpec 
		model: self;
		list: #changeSetList; 
		getSelected: #currentCngSet; 
		setSelected: #showChangeSetNamed:; 
		menu: #changeSetMenu:shifted:; 
		keyPress: #changeSetListKey:from:;
		autoDeselect: false;
		frame: (((0@0 extent: 0.5@csListHeight)
			scaleBy: rect extent) translateBy: rect origin).
	window children add: listSpec.

	listSpec := builder pluggableListSpec new.
	listSpec 
		model: self;
		list: #classList; 
		getSelected: #currentClassName; 
		setSelected: #currentClassName:; 
		menu: #classListMenu:shifted:; 
		keyPress: #classListKey:from:;
		frame: (((0.5@0 extent: 0.5@csListHeight)
			scaleBy: rect extent) translateBy: rect origin).
	window children add: listSpec.

	listSpec := builder pluggableListSpec new.
	listSpec 
		model: self;
		list: #messageList; 
		getSelected: #currentSelector;
		setSelected: #currentSelector:; 
		menu: #messageMenu:shifted:; 
		keyPress: #messageListKey:from:;
		frame: (((0@csListHeight extent: 1@msgListHeight)
			scaleBy: rect extent) translateBy: rect origin).
	window children add: listSpec.

	textSpec := builder pluggableTextSpec new.
	textSpec 
		model: self;
		getText: #contents; 
		setText: #contents:notifying:; 
		selection: #contentsSelection; 
		menu: #codePaneMenu:shifted:;
		frame: (((0@csMsgListHeight corner: 1@1) scaleBy: rect extent) translateBy: rect origin).
	window children add: textSpec.
	^window! !

!TranscriptStream class methodsFor: '*ToolBuilder-Specs' stamp: 'ar 2/11/2005 18:10'!
buildWith: aBuilder
	^self new buildWith: aBuilder! !

!TranscriptStream methodsFor: '*ToolBuilder-Specs' stamp: 'ar 2/11/2005 20:36'!
buildWith: builder
	| windowSpec textSpec |
	windowSpec := builder pluggableWindowSpec new.
	windowSpec model: self.
	windowSpec label: 'Transcript'.
	windowSpec children: OrderedCollection new.

	textSpec := builder pluggableTextSpec new.
	textSpec 
		model: self;
		menu: #codePaneMenu:shifted:;
		frame: (0@0corner: 1@1).
	windowSpec children add: textSpec.

	^builder build: windowSpec! !
